/**
 * Crypto Class 1.1
 *
 * @author Valentin Schmidt
 * @version 1.1
 *
 * Simple string encryption/decryption functions that are compatible with
 * the Lingo function counterparts in script 'crypto.lib.ls'
 *
 * Crypto Xtra AES specs:
 * - AES-128 Encryption
 * - Mode: CBC
 * - IV prepended to encrypted data
 * - optional 12 byte header starts with 'AES', byte 5 contains pad_length
 *
 */

using System;
using System.Text;
using System.IO;
using System.Security.Cryptography;

class Crypto{

  public const int AES_HEADER_LEN = 12;
  public const int AES_BLOCK_LEN = 16;
  public const int AES_KEY_LEN = 16; // 16 for AES-128
  public const int AES_IV_LEN = 16;

  /**
    * @param string input 
    * @param string key 16-32 bytes (alternatively use raw md5 checksum)
    * @param bool add_header Optional, default=false
    *
    * @return string Returns encrypted string (Base64 encoded)
    */
  static string encrypt_string(string input, string key, bool add_header = false)
  {

    // Check arguments. 
    if (input == null || input.Length <= 0) throw new ArgumentNullException("input");
    if (key == null || key.Length<AES_KEY_LEN || key.Length>2*AES_KEY_LEN) throw new ArgumentNullException("key");

    byte[] encrypted;

    // Create a new instance of the RijndaelManaged class for AES-128 encryption
    using (RijndaelManaged aes = new RijndaelManaged())
    {
      //aes.Mode = CipherMode.CBC; // =default!
      aes.Padding = PaddingMode.Zeros;
      aes.Key = Encoding.ASCII.GetBytes(key);
      //aes.Key = MD5.Create().ComputeHash(Encoding.ASCII.GetBytes(key)); // optional, to allow arbitrary key length
      aes.GenerateIV(); // generates a new initialization vector (IV).

      // Encrypt the string to an array of bytes. 
      ICryptoTransform encryptor = aes.CreateEncryptor(aes.Key, aes.IV);
      using (MemoryStream msEncrypt = new MemoryStream())
      {
        using (CryptoStream csEncrypt = new CryptoStream(msEncrypt, encryptor, CryptoStreamMode.Write))
        {
          using (StreamWriter swEncrypt = new StreamWriter(csEncrypt))
          {
            swEncrypt.Write(input);
          }
          encrypted = msEncrypt.ToArray();
        }
      }

      int data_start = add_header ? AES_HEADER_LEN : 0;
      byte[] concat = new byte[data_start + aes.IV.Length + encrypted.Length]; 

      // add header
      if (add_header) {
        int pad_len = AES_BLOCK_LEN - (input.Length % AES_BLOCK_LEN);
        byte[] header = new byte[AES_HEADER_LEN] { 65, 69, 83, (byte)AES_KEY_LEN, 0, (byte)pad_len, 0, 0, 0, 0, 0, 0 };
        System.Buffer.BlockCopy(header, 0, concat, 0, header.Length);
      }

      System.Buffer.BlockCopy(aes.IV, 0, concat, data_start, aes.IV.Length);
      System.Buffer.BlockCopy(encrypted, 0, concat, data_start+aes.IV.Length, encrypted.Length);

      return Convert.ToBase64String(concat);
    }
  }


  /**
   * @param string input Encrypted string (Base64 encoded)
   * @param string key 16-32 bytes (alternatively use raw md5 checksum)
   *
   * @return string Returns decrypted string
   */
  static string decrypt_string(string input, string key){
    // Check arguments. 
    if (input == null || input.Length <= 0) throw new ArgumentNullException("input");
    if (key == null || key.Length<AES_KEY_LEN || key.Length>2*AES_KEY_LEN) throw new ArgumentNullException("key");

    byte[] inputBytes = Convert.FromBase64String(input);

    int pad_len = 0;
    int data_start = 0;

    // parse header
    if (inputBytes[0] == 65 && inputBytes[1] == 69 && inputBytes[2] == 83)//"AES"=65, 69, 83
    {
      pad_len = (int)inputBytes[5];
      data_start = AES_HEADER_LEN;
    }

    string decrypted = "";

    // Create a new instance of the RijndaelManaged class for AES-128 encryption
    using (RijndaelManaged aes = new RijndaelManaged())
    {
      //aes.Mode = CipherMode.CBC; // =default!
      aes.Padding = PaddingMode.Zeros;

      aes.Key = Encoding.ASCII.GetBytes(key);
      //aes.Key = MD5.Create().ComputeHash(Encoding.ASCII.GetBytes(key)); // optional, to allow arbitrary key length

      byte[] iv = new byte[AES_IV_LEN];
      System.Buffer.BlockCopy(inputBytes, data_start, iv, 0, AES_IV_LEN);
      aes.IV = iv;

      byte[] encrypted = new byte[inputBytes.Length - data_start - AES_IV_LEN];
      System.Buffer.BlockCopy(inputBytes, data_start + AES_IV_LEN, encrypted, 0, inputBytes.Length-data_start-AES_IV_LEN);

      // Create a decrytor to perform the stream transform.
      ICryptoTransform decryptor = aes.CreateDecryptor(aes.Key, aes.IV);

      // Create the streams used for decryption. 
      using (MemoryStream msDecrypt = new MemoryStream(encrypted))
      {
        using (CryptoStream csDecrypt = new CryptoStream(msDecrypt, decryptor, CryptoStreamMode.Read))
        {
          using (StreamReader srDecrypt = new StreamReader(csDecrypt))
          {
            decrypted = srDecrypt.ReadToEnd();
          }
        }
      }
    }

    if (pad_len > 0) decrypted = decrypted.Substring(0, decrypted.Length - pad_len);
    return decrypted;
  } 

}